'use strict'
/**
 * A fellow with similarties to other people.
 * @property {string} homepage
 * @property {string} githubUrl
 * @property {string} githubUsername
 * @property {string} twitterUrl
 * @property {string} twitterUsername
 * @property {string} facebookUrl
 * @property {string} facebookUsername
 * @class
 */

function _typeof(obj) {
	'@babel/helpers - typeof'
	if (typeof Symbol === 'function' && typeof Symbol.iterator === 'symbol') {
		_typeof = function _typeof(obj) {
			return typeof obj
		}
	} else {
		_typeof = function _typeof(obj) {
			return obj &&
				typeof Symbol === 'function' &&
				obj.constructor === Symbol &&
				obj !== Symbol.prototype
				? 'symbol'
				: typeof obj
		}
	}
	return _typeof(obj)
}

function _createForOfIteratorHelper(o) {
	if (typeof Symbol === 'undefined' || o[Symbol.iterator] == null) {
		if (Array.isArray(o) || (o = _unsupportedIterableToArray(o))) {
			var i = 0
			var F = function F() {}
			return {
				s: F,
				n: function n() {
					if (i >= o.length) return { done: true }
					return { done: false, value: o[i++] }
				},
				e: function e(_e) {
					throw _e
				},
				f: F,
			}
		}
		throw new TypeError(
			'Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.'
		)
	}
	var it,
		normalCompletion = true,
		didErr = false,
		err
	return {
		s: function s() {
			it = o[Symbol.iterator]()
		},
		n: function n() {
			var step = it.next()
			normalCompletion = step.done
			return step
		},
		e: function e(_e2) {
			didErr = true
			err = _e2
		},
		f: function f() {
			try {
				if (!normalCompletion && it.return != null) it.return()
			} finally {
				if (didErr) throw err
			}
		},
	}
}

function _unsupportedIterableToArray(o, minLen) {
	if (!o) return
	if (typeof o === 'string') return _arrayLikeToArray(o, minLen)
	var n = Object.prototype.toString.call(o).slice(8, -1)
	if (n === 'Object' && o.constructor) n = o.constructor.name
	if (n === 'Map' || n === 'Set') return Array.from(n)
	if (n === 'Arguments' || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))
		return _arrayLikeToArray(o, minLen)
}

function _arrayLikeToArray(arr, len) {
	if (len == null || len > arr.length) len = arr.length
	for (var i = 0, arr2 = new Array(len); i < len; i++) {
		arr2[i] = arr[i]
	}
	return arr2
}

function _classCallCheck(instance, Constructor) {
	if (!(instance instanceof Constructor)) {
		throw new TypeError('Cannot call a class as a function')
	}
}

function _defineProperties(target, props) {
	for (var i = 0; i < props.length; i++) {
		var descriptor = props[i]
		descriptor.enumerable = descriptor.enumerable || false
		descriptor.configurable = true
		if ('value' in descriptor) descriptor.writable = true
		Object.defineProperty(target, descriptor.key, descriptor)
	}
}

function _createClass(Constructor, protoProps, staticProps) {
	if (protoProps) _defineProperties(Constructor.prototype, protoProps)
	if (staticProps) _defineProperties(Constructor, staticProps)
	return Constructor
}

var Fellow = /*#__PURE__*/ (function () {
	_createClass(
		Fellow,
		[
			{
				key: 'compare',

				/**
    Compare a fellow to another fellow, uses ensureFields for the comparison
    If an ensureField is an array for both ourself and the other fellow, we will check to see if only one item of that array is similar (this is useful for email comparisons)
    @param {Fellow} other The other fellow to compare ourselves with
    @returns {Boolean} Returns `true` if they appear to be the same person, or `false` if not
    */
				value: function compare(other) {
					var fields = this.ensureFields

					var _iterator = _createForOfIteratorHelper(fields),
						_step

					try {
						for (_iterator.s(); !(_step = _iterator.n()).done; ) {
							var field = _step.value
							var value = this[field]
							var otherValue = other[field]

							if (value && otherValue) {
								if (Array.isArray(value) && Array.isArray(otherValue)) {
									var _iterator2 = _createForOfIteratorHelper(value),
										_step2

									try {
										for (_iterator2.s(); !(_step2 = _iterator2.n()).done; ) {
											var item = _step2.value

											if (otherValue.indexOf(item) !== -1) {
												return true
											}
										}
									} catch (err) {
										_iterator2.e(err)
									} finally {
										_iterator2.f()
									}
								} else if (value === otherValue) {
									return true
								}
							}
						}
					} catch (err) {
						_iterator.e(err)
					} finally {
						_iterator.f()
					}

					return false
				},
				/**
    Get a fellow from the singleton list
    @method get
    @param {*} value The value to fetch the value with
    @returns {Fellow|Null} The fetched fellow, if they exist with that value
    */
			},
			{
				key: 'ensureFields',

				/**
    An array of fields that will denote if the user is the same user or not when using ensure
    @access protected
    @type {Array}
    */
				get: function get() {
					return [
						'emails',
						'homepage',
						'githubUsername',
						'twitterUsername',
						'facebookUsername',
					]
				},
			},
			{
				key: 'json',
				// -----------------------------------
				// Properties

				/**
    Get the serialisable values of this fellow
    @type {Object}
    */
				get: function get() {
					var json = JSON.parse(JSON.stringify(this))
					return json
				},
				/**
    When fetching,
    Updating the email will add the email to the start of the `emails` array.
    	When getting,
    Fetches the first email from the emails array.
    @property {String} email
    @param {String} value - The email to add to the emails
    */
			},
			{
				key: 'email',
				set: function set(value) {
					var index = this.emails.indexOf(value)

					if (index !== -1) {
						this.emails = this.emails
							.slice(0, index)
							.concat(this.emails.slice(index + 1))
					}

					this.emails.unshift(value)
				},
				get: function get() {
					return this.emails[0] || null
				},
				/**
    When setting,
    Will determine passed url is a github, facebook, or twitter url.
    If it is, then it will extract the username and url from it.
    If this is not one of those urls, then it will set the homepage variable.
    	When getting,
    Will fetch `homepage || githubUrl || facebookUrl || twitterUrl || null`
    @type {String}
    @param {String} value
    @returns {this}
    */
			},
			{
				key: 'url',
				set: function set(value) {
					var githubMatch = /^.+github.com\/([^/]+)\/?$/.exec(value)

					if (githubMatch) {
						this.githubUsername = githubMatch[1]
						this.githubUrl = 'https://github.com/' + this.githubUsername
					} else {
						var facebookMatch = /^.+facebook.com\/([^/]+)\/?$/.exec(value)

						if (facebookMatch) {
							this.facebookUsername = facebookMatch[1]
							this.facebookUrl = 'https://facebook.com/' + this.facebookUsername
						} else {
							var twitterMatch = /^.+twitter.com\/([^/]+)\/?$/.exec(value)

							if (twitterMatch) {
								this.twitterUsername = twitterMatch[1]
								this.twitterUrl = 'https://twitter.com/' + this.twitterUsername
							} else {
								this.homepage = value.replace(/^https/, 'http')
							}
						}
					}
				},
				get: function get() {
					return (
						this.homepage ||
						this.githubUrl ||
						this.facebookUrl ||
						this.twitterUrl ||
						null
					)
				}, // -----------------------------------
				// Methods

				/**
    Construct our fellow instance with the value
    @param {*} value - The value used to set the properties of the fellow, forwarded to set
    */
			},
		],
		[
			{
				key: 'create',
				// -----------------------------------
				// Static Helpers

				/**
    Create a new Fellow instance with the value, however if the value is already a fellow instance, then just return it
    @static
    @param {*} value
    @returns {Fellow} The new fellow instance
    */
				value: function create(value) {
					return value instanceof this ? value : new this(value)
				},
				/**
    A singleton array attached to the class object that stores it's people objects
    @static
    @type {Array}
    */
			},
			{
				key: 'ensure',

				/**
    With the value, see if an existing fellow exists in our singleton list property with the value, otherwise create a new fellow instance with the value and add them to our singleton list
    @static
    @param {*} value - The value to create a new fellow instance or find the existing fellow instance with
    @param {boolean} [add=true] - Whether to add the created person to the list
    @returns {Fellow} The new or existing fellow instance
    */
				value: function ensure(value) {
					var add =
						arguments.length > 1 && arguments[1] !== undefined
							? arguments[1]
							: true
					var newFellow = this.create(value)
					var people = this.list

					var _iterator3 = _createForOfIteratorHelper(people),
						_step3

					try {
						for (_iterator3.s(); !(_step3 = _iterator3.n()).done; ) {
							var existingFellow = _step3.value

							if (newFellow.compare(existingFellow)) {
								return existingFellow.set(value)
							}
						}
					} catch (err) {
						_iterator3.e(err)
					} finally {
						_iterator3.f()
					}

					if (add) {
						people.push(newFellow)
						return newFellow
					} else {
						return null
					}
				},
			},
			{
				key: 'get',
				value: function get(value) {
					return this.ensure(value, false)
				},
				/**
    Add a fellow or a series of people, denoted by the value, to the singleton list
    @static
    @param {*} value The fellow or people to add
    @returns {Array} An array of the fellow objects for the passed people
    */
			},
			{
				key: 'add',
				value: function add(value) {
					var _this = this

					if (value instanceof this) {
						return [this.ensure(value)]
					} else if (typeof value === 'string') {
						return value.split(/, +/).map(function (fellow) {
							return _this.ensure(fellow)
						})
					} else if (Array.isArray(value)) {
						return value.map(function (value) {
							return _this.ensure(value)
						})
					} else if (value) {
						return [this.ensure(value)]
					} else {
						return []
					}
				},
				/**
    Get all fellows who contribute to a particular repository
    @static
    @param {String} repoSlug The repository slug
    @returns {Array} An array of the fellow objects that contribute to the repository
    */
			},
			{
				key: 'contributesRepository',
				value: function contributesRepository(repoSlug) {
					return this.list.filter(function (fellow) {
						return fellow.ensureRepository(repoSlug).contributes
					})
				},
				/**
    Get all fellows who maintain a particular repository
    @static
    @param {String} repoSlug The repository slug
    @returns {Array} An array of the fellow objects that maintain to the repository
    */
			},
			{
				key: 'maintainsRepository',
				value: function maintainsRepository(repoSlug) {
					return this.list.filter(function (fellow) {
						return fellow.ensureRepository(repoSlug).maintains
					})
				},
				/**
    Get all fellows who author a particular repository
    @static
    @param {String} repoSlug The repository slug
    @returns {Array} An array of the fellow objects that author to the repository
    */
			},
			{
				key: 'authorsRepository',
				value: function authorsRepository(repoSlug) {
					return this.list.filter(function (fellow) {
						return fellow.ensureRepository(repoSlug).authors
					})
				},
			},
			{
				key: 'list',
				get: function get() {
					if (this._list == null) this._list = []
					return this._list
				},
			},
		]
	)

	function Fellow(value) {
		_classCallCheck(this, Fellow)

		if (!this.emails) this.emails = []
		this.set(value)
	}
	/**
  Update our fellow with the passed value
  @param {String|Object} fellow - A string in the format of "Benjamin Lupton <b@lupton.cc> (https://balupton.com)" or an object with properties
  @returns {this}
  */

	_createClass(Fellow, [
		{
			key: 'set',
			value: function set(fellow) {
				var _this2 = this

				// String format
				// Benjamin Lupton <b@lupton.cc> (https://balupton.com)
				if (typeof fellow === 'string') {
					var match = /^([^<(]+)\s*(?:<(.+?)>)?\s*(?:\((.+?)\))?$/.exec(fellow)

					if (!match) {
						throw new Error('Invalid fellow string')
					}

					var name = (match[1] || '').trim() || null
					var email = (match[2] || '').trim() || null
					var url = (match[3] || '').trim() || null
					if (name) this.name = name
					if (email) this.email = email
					if (url) this.url = url
				} // Object Format
				else if (_typeof(fellow) === 'object') {
					var urlFields = [
						'url',
						'homepage',
						'web',
						'githubUrl',
						'twitterUrl',
						'facebookUrl',
					]
					Object.keys(fellow).forEach(function (key) {
						if (key[0] === '_') return // skip if private

						var value = fellow[key] || null

						if (value) {
							// if not a url field, e.g. name or email
							if (urlFields.indexOf(key) === -1) {
								_this2[key] = value
							} // if any of the url fields, redirect to url setter
							else {
								_this2.url = value
							}
						}
					})
				} else {
					throw new Error('Invalid fellow input')
				}

				return this
			}, // -----------------------------------
			// Repos

			/**
    Ensures that the github repository exists on the class
    @private
    @param {String} slug - Github repository slug (e.g. "bevry/projectz")
    @returns {Object} The repository object
    */
		},
		{
			key: 'ensureRepository',
			value: function ensureRepository(slug) {
				if (this._Repositories == null) this._Repositories = {}
				if (this._Repositories[slug] == null)
					this._Repositories[slug] = {
						contributes: false,
						maintains: false,
						authors: false,
					}
				return this._Repositories[slug]
			},
			/**
    Get all added github repository slugs that this fellow contributes to
    @type {Array}
    */
		},
		{
			key: 'contributesRepository',

			/**
    Make note that this fellow contributes to this repository slug
    @param {String} slug - The github repository slug that this user contributes to
    @returns {this}
    */
			value: function contributesRepository(slug) {
				this.ensureRepository(slug).contributes = true
				return this
			},
			/**
    Get all added github repository slugs that this fellow maintains
    @type {Array}
    */
		},
		{
			key: 'maintainsRepository',

			/**
    Make note that this fellow maintains this repository slug
    @param {String} slug - The github repository slug that this user maintains
    @returns {this}
    */
			value: function maintainsRepository(slug) {
				this.ensureRepository(slug).maintains = true
				return this
			},
			/**
    Get all added github repository slugs that this fellow authors
    @type {Array}
    */
		},
		{
			key: 'authorsRepository',

			/**
    Make note that this fellow authors this repository slug
    @param {String} slug - The github repository slug that this user authors
    @returns {this}
    */
			value: function authorsRepository(slug) {
				this.ensureRepository(slug).authors = true
				return this
			},
		},
		{
			key: 'contributedRepositories',
			get: function get() {
				var _this3 = this

				return Object.keys(this._Repositories || {}).filter(function (slug) {
					var repo = _this3._Repositories[slug]
					return repo && repo.contributes
				})
			},
		},
		{
			key: 'maintainedRepositories',
			get: function get() {
				var _this4 = this

				return Object.keys(this._Repositories || {}).filter(function (slug) {
					var repo = _this4._Repositories[slug]
					return repo && repo.maintains
				})
			},
		},
		{
			key: 'authoredRepositories',
			get: function get() {
				var _this5 = this

				return Object.keys(this._Repositories || {}).filter(function (slug) {
					var repo = _this5._Repositories[slug]
					return repo && repo.authors
				})
			},
		},
	])

	return Fellow
})() // Export

module.exports = Fellow
